﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Automation;
using System.Diagnostics;
using System.Collections.ObjectModel;
using System.Globalization;

namespace Calculator
{
    class Program
    {
        static void Main(string[] args)
        {
            string argument = args[0];

            // Create a window open waiter and wait for a window to open with correct title            
            WindowControl calc = null;
            using (WindowOpenWaiter waiter = new WindowOpenWaiter("Calculator"))
            {
                try
                {
                    waiter.Wait(1000 * 10);
                }
                catch (TimeoutException e)
                {
                    Console.WriteLine(e.Message);
                    return;
                }

                calc = new WindowControl(waiter.Element);
            }

            // Create automatin element for some buttons in calc
            AutomationElement edit = GetElement(calc.Element, "", "Edit");
            AutomationElement plus = GetElement(calc.Element, "+", "Button");
            AutomationElement minus = GetElement(calc.Element, "-", "Button");
            AutomationElement div = GetElement(calc.Element, "/", "Button");
            AutomationElement mult = GetElement(calc.Element, "*", "Button");
            AutomationElement egual = GetElement(calc.Element, "=", "Button");

            // Create a list of number buttons in calc
            Collection<AutomationElement> numberButtons = new Collection<AutomationElement>();
            for (int i = 0; i < 10; i++)
            {
                numberButtons.Insert(i, GetElement(calc.Element, i.ToString(CultureInfo.CurrentCulture), "Button"));
            }

            // Read input data and press according button
            foreach (char ch in argument.ToCharArray())
            {
                switch (ch)
                {
                    case '0': GetInvokePattern(numberButtons[0]).Invoke();
                        break;
                    case '1': GetInvokePattern(numberButtons[1]).Invoke();
                        break;
                    case '2': GetInvokePattern(numberButtons[2]).Invoke();
                        break;
                    case '3': GetInvokePattern(numberButtons[3]).Invoke();
                        break;
                    case '4': GetInvokePattern(numberButtons[4]).Invoke();
                        break;
                    case '5': GetInvokePattern(numberButtons[5]).Invoke();
                        break;
                    case '6': GetInvokePattern(numberButtons[6]).Invoke();
                        break;
                    case '7': GetInvokePattern(numberButtons[7]).Invoke();
                        break;
                    case '8': GetInvokePattern(numberButtons[8]).Invoke();
                        break;
                    case '9': GetInvokePattern(numberButtons[9]).Invoke();
                        break;
                    case '+': GetInvokePattern(plus).Invoke();
                        break;
                    case '-': GetInvokePattern(minus).Invoke();
                        break;
                    case '/': GetInvokePattern(div).Invoke();
                        break;
                    case '*': GetInvokePattern(mult).Invoke();
                        break;
                    default:
                        break;
                }

                // Some sleep so you can see what happening with calc
                System.Threading.Thread.Sleep(200);
            }

            // Press equal
            GetInvokePattern(egual).Invoke();

            // Get the result from the input field
            Console.WriteLine(argument + "=" + GetValue(edit));

            // Close window
            calc.CloseWindowByXButton();            
            Console.ReadKey();
        }

        /// <summary>
        /// Return an automation element based on some search criterias
        /// </summary>
        /// <param name="root">Where to start to search from</param>
        /// <param name="name">The name of the element to search for</param>
        /// <param name="className">The class name of the element to search for</param>
        /// <returns>The firt automation element found that fullfil the search criterias</returns>
        private static AutomationElement GetElement(AutomationElement root, string name, string className)
        {
            PropertyCondition condName = new PropertyCondition(AutomationElement.NameProperty, name);
            PropertyCondition condClassName = new PropertyCondition(AutomationElement.ClassNameProperty, className);
            AndCondition cond = new AndCondition(condName, condClassName);
            return root.FindFirst(TreeScope.Descendants, cond);
        }

        /// <summary>
        /// The the invoke pattarn for a specific automation element
        /// </summary>
        /// <param name="element">The element to get invoke pattern for</param>
        /// <returns>The invoke pattern</returns>
        private static InvokePattern GetInvokePattern(AutomationElement element)
        {
            return element.GetCurrentPattern(InvokePattern.Pattern) as InvokePattern;
        }

        /// <summary>
        /// Get the value for an automation element
        /// </summary>
        /// <param name="element">The element to get value for</param>
        /// <returns>The value</returns>
        private static string GetValue(AutomationElement element)
        {
            ValuePattern value = element.GetCurrentPattern(ValuePattern.Pattern) as ValuePattern;
            return value.Current.Value;
        }
    }
}