﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Automation;

namespace Calculator
{
    /// <summary>
    /// Window control is an abstraction for a window element
    /// </summary>
    public class WindowControl
    {
        private AutomationElement element;
        private WindowPattern windowPattern;

        /// <summary>
        /// The AutomationElement that the WindowControl is based on
        /// </summary>
        public AutomationElement Element
        {
            get
            {
                return this.element;
            }
        }

        /// <summary>
        /// Window constrol constructor
        /// </summary>
        /// <param name="element">AutomationElement for the window</param>
        public WindowControl(AutomationElement element)
        {
            if(element == null)
            {
                throw new ArgumentNullException("element");
            }

            this.element = element;
            this.windowPattern = element.GetCurrentPattern(WindowPattern.Pattern) as WindowPattern;
        }

        /// <summary>
        /// Close the window
        /// </summary>
        public void CloseWindow()
        {
            this.windowPattern.Close();
        }

        /// <summary>
        /// Will set focus to the window
        /// </summary>
        public void SetFocus()
        {
            this.element.SetFocus();
        }

        /// <summary>
        /// Will use the litle X button in the titelbar to close the window
        /// </summary>
        public void CloseWindowByXButton()
        {
            // Get the title bar
            PropertyCondition condTitelbar = new PropertyCondition(AutomationElement.ControlTypeProperty, ControlType.TitleBar);
            AutomationElement titelbar = element.FindFirst(TreeScope.Descendants, condTitelbar);

            // Get the close button
            PropertyCondition condButton = new PropertyCondition(AutomationElement.ControlTypeProperty, ControlType.Button);
            PropertyCondition condName = new PropertyCondition(AutomationElement.NameProperty, "Close");
            AndCondition andCond = new AndCondition(condButton, condName);
            AutomationElement closeButton = titelbar.FindFirst(TreeScope.Descendants, andCond);

            // Invoke the close button
            (closeButton.GetCurrentPattern(InvokePattern.Pattern) as InvokePattern).Invoke();
        }
    }
}
